@@ -5,16 +5,16 @@ mod helpers;
 use {
     bincode::deserialize,
     helpers::*,
-    solana_program::{
-        clock::Epoch, hash::Hash, instruction::InstructionError, pubkey::Pubkey,
-        system_instruction::SystemError,
-    },
+    solana_program::{clock::Epoch, hash::Hash, instruction::InstructionError, pubkey::Pubkey},
     solana_program_test::*,
     solana_sdk::{
         signature::{Keypair, Signer},
         transaction::{Transaction, TransactionError},
     },
-    spl_stake_pool::{error::StakePoolError, id, instruction, stake_program},
+    spl_stake_pool::{
+        error::StakePoolError, find_transient_stake_program_address, id, instruction,
+        stake_program, MINIMUM_ACTIVE_STAKE,
+    },
 };
 
 async fn setup() -> (
@@ -46,7 +46,7 @@ async fn setup() -> (
     )
     .await;
 
-    let _deposit_info = simple_deposit(
+    let _deposit_info = simple_deposit_stake(
         &mut banks_client,
         &payer,
         &recent_blockhash,
@@ -103,6 +103,7 @@ async fn success() {
             &validator_stake.transient_stake_account,
             &validator_stake.vote.pubkey(),
             increase_amount,
+            validator_stake.transient_stake_seed,
         )
         .await;
     assert!(error.is_none());
@@ -160,6 +161,7 @@ async fn fail_with_wrong_withdraw_authority() {
             &validator_stake.transient_stake_account,
             &validator_stake.vote.pubkey(),
             reserve_lamports / 2,
+            validator_stake.transient_stake_seed,
         )],
         Some(&payer.pubkey()),
         &[&payer, &stake_pool_accounts.staker],
@@ -205,6 +207,7 @@ async fn fail_with_wrong_validator_list() {
             &validator_stake.transient_stake_account,
             &validator_stake.vote.pubkey(),
             reserve_lamports / 2,
+            validator_stake.transient_stake_seed,
         )],
         Some(&payer.pubkey()),
         &[&payer, &stake_pool_accounts.staker],
@@ -237,15 +240,13 @@ async fn fail_with_unknown_validator() {
         reserve_lamports,
     ) = setup().await;
 
-    let unknown_stake = ValidatorStakeAccount::new(&stake_pool_accounts.stake_pool.pubkey());
-    unknown_stake
-        .create_and_delegate(
-            &mut banks_client,
-            &payer,
-            &recent_blockhash,
-            &stake_pool_accounts.staker,
-        )
-        .await;
+    let unknown_stake = create_unknown_validator_stake(
+        &mut banks_client,
+        &payer,
+        &recent_blockhash,
+        &stake_pool_accounts.stake_pool.pubkey(),
+    )
+    .await;
 
     let transaction = Transaction::new_signed_with_payer(
         &[instruction::increase_validator_stake(
@@ -258,6 +259,7 @@ async fn fail_with_unknown_validator() {
             &unknown_stake.transient_stake_account,
             &unknown_stake.vote.pubkey(),
             reserve_lamports / 2,
+            unknown_stake.transient_stake_seed,
         )],
         Some(&payer.pubkey()),
         &[&payer, &stake_pool_accounts.staker],
@@ -270,13 +272,13 @@ async fn fail_with_unknown_validator() {
         .unwrap()
         .unwrap();
 
-    match error {
-        TransactionError::InstructionError(_, InstructionError::Custom(error_index)) => {
-            let program_error = StakePoolError::ValidatorNotFound as u32;
-            assert_eq!(error_index, program_error);
-        }
-        _ => panic!("Wrong error"),
-    }
+    assert_eq!(
+        error,
+        TransactionError::InstructionError(
+            0,
+            InstructionError::Custom(StakePoolError::ValidatorNotFound as u32)
+        )
+    );
 }
 
 #[tokio::test]
@@ -298,25 +300,35 @@ async fn fail_increase_twice() {
             &validator_stake.transient_stake_account,
             &validator_stake.vote.pubkey(),
             reserve_lamports / 3,
+            validator_stake.transient_stake_seed,
         )
         .await;
     assert!(error.is_none());
 
+    let transient_stake_seed = validator_stake.transient_stake_seed * 100;
+    let transient_stake_address = find_transient_stake_program_address(
+        &id(),
+        &validator_stake.vote.pubkey(),
+        &stake_pool_accounts.stake_pool.pubkey(),
+        transient_stake_seed,
+    )
+    .0;
     let error = stake_pool_accounts
         .increase_validator_stake(
             &mut banks_client,
             &payer,
             &recent_blockhash,
-            &validator_stake.transient_stake_account,
-            &validator_stake.vote.pubkey(),
+            &validator_stake.stake_account,
+            &transient_stake_address,
             reserve_lamports / 4,
+            transient_stake_seed,
         )
         .await
         .unwrap()
         .unwrap();
     match error {
         TransactionError::InstructionError(_, InstructionError::Custom(error_index)) => {
-            let program_error = SystemError::AccountAlreadyInUse as u32;
+            let program_error = StakePoolError::ValidatorNotFound as u32;
             assert_eq!(error_index, program_error);
         }
         _ => panic!("Wrong error"),
@@ -334,9 +346,6 @@ async fn fail_with_small_lamport_amount() {
         _reserve_lamports,
     ) = setup().await;
 
-    let rent = banks_client.get_rent().await.unwrap();
-    let stake_rent = rent.minimum_balance(std::mem::size_of::<stake_program::StakeState>());
-
     let error = stake_pool_accounts
         .increase_validator_stake(
             &mut banks_client,
@@ -344,7 +353,8 @@ async fn fail_with_small_lamport_amount() {
             &recent_blockhash,
             &validator_stake.transient_stake_account,
             &validator_stake.vote.pubkey(),
-            stake_rent,
+            MINIMUM_ACTIVE_STAKE - 1,
+            validator_stake.transient_stake_seed,
         )
         .await
         .unwrap()
@@ -375,6 +385,7 @@ async fn fail_overdraw_reserve() {
             &validator_stake.transient_stake_account,
             &validator_stake.vote.pubkey(),
             reserve_lamports,
+            validator_stake.transient_stake_seed,
         )
         .await
         .unwrap()
