@@ -11,7 +11,7 @@ use {
         hash::Hash,
         instruction::{AccountMeta, Instruction, InstructionError},
         pubkey::Pubkey,
-        sysvar,
+        system_program, sysvar,
     },
     solana_program_test::*,
     solana_sdk::{
@@ -19,7 +19,9 @@ use {
         transaction::{Transaction, TransactionError},
         transport::TransportError,
     },
-    spl_stake_pool::{error::StakePoolError, id, instruction, stake_program, state},
+    spl_stake_pool::{
+        error::StakePoolError, find_stake_program_address, id, instruction, stake_program, state,
+    },
 };
 
 async fn setup() -> (
@@ -36,28 +38,28 @@ async fn setup() -> (
         .await
         .unwrap();
 
-    let user_stake = ValidatorStakeAccount::new(&stake_pool_accounts.stake_pool.pubkey());
-    user_stake
-        .create_and_delegate(
-            &mut banks_client,
-            &payer,
-            &recent_blockhash,
-            &stake_pool_accounts.staker,
-        )
-        .await;
+    let validator_stake = ValidatorStakeAccount::new(&stake_pool_accounts.stake_pool.pubkey(), 0);
+    create_vote(
+        &mut banks_client,
+        &payer,
+        &recent_blockhash,
+        &validator_stake.validator,
+        &validator_stake.vote,
+    )
+    .await;
 
     (
         banks_client,
         payer,
         recent_blockhash,
         stake_pool_accounts,
-        user_stake,
+        validator_stake,
     )
 }
 
 #[tokio::test]
 async fn success() {
-    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, user_stake) =
+    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, validator_stake) =
         setup().await;
 
     let error = stake_pool_accounts
@@ -65,7 +67,8 @@ async fn success() {
             &mut banks_client,
             &payer,
             &recent_blockhash,
-            &user_stake.stake_account,
+            &validator_stake.stake_account,
+            &validator_stake.vote.pubkey(),
         )
         .await;
     assert!(error.is_none());
@@ -87,16 +90,18 @@ async fn success() {
             },
             validators: vec![state::ValidatorStakeInfo {
                 status: state::StakeStatus::Active,
-                vote_account_address: user_stake.vote.pubkey(),
+                vote_account_address: validator_stake.vote.pubkey(),
                 last_update_epoch: 0,
                 active_stake_lamports: 0,
                 transient_stake_lamports: 0,
+                transient_seed_suffix_start: 0,
+                transient_seed_suffix_end: 0,
             }]
         }
     );
 
-    // Check of stake account authority has changed
-    let stake = get_account(&mut banks_client, &user_stake.stake_account).await;
+    // Check stake account existence and authority
+    let stake = get_account(&mut banks_client, &validator_stake.stake_account).await;
     let stake_state = deserialize::<stake_program::StakeState>(&stake.data).unwrap();
     match stake_state {
         stake_program::StakeState::Stake(meta, _) => {
@@ -115,7 +120,7 @@ async fn success() {
 
 #[tokio::test]
 async fn fail_with_wrong_validator_list_account() {
-    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, user_stake) =
+    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, validator_stake) =
         setup().await;
 
     let wrong_validator_list = Keypair::new();
@@ -125,9 +130,11 @@ async fn fail_with_wrong_validator_list_account() {
             &id(),
             &stake_pool_accounts.stake_pool.pubkey(),
             &stake_pool_accounts.staker.pubkey(),
+            &payer.pubkey(),
             &stake_pool_accounts.withdraw_authority,
             &wrong_validator_list.pubkey(),
-            &user_stake.stake_account,
+            &validator_stake.stake_account,
+            &validator_stake.vote.pubkey(),
         )],
         Some(&payer.pubkey()),
     );
@@ -150,122 +157,9 @@ async fn fail_with_wrong_validator_list_account() {
     }
 }
 
-#[tokio::test]
-async fn fail_too_little_stake() {
-    let (mut banks_client, payer, recent_blockhash) = program_test().start().await;
-    let stake_pool_accounts = StakePoolAccounts::new();
-    stake_pool_accounts
-        .initialize_stake_pool(&mut banks_client, &payer, &recent_blockhash, 1)
-        .await
-        .unwrap();
-
-    let user_stake = ValidatorStakeAccount::new(&stake_pool_accounts.stake_pool.pubkey());
-    create_vote(
-        &mut banks_client,
-        &payer,
-        &recent_blockhash,
-        &user_stake.validator,
-        &user_stake.vote,
-    )
-    .await;
-
-    create_validator_stake_account(
-        &mut banks_client,
-        &payer,
-        &recent_blockhash,
-        &user_stake.stake_pool,
-        &stake_pool_accounts.staker,
-        &user_stake.stake_account,
-        &user_stake.vote.pubkey(),
-    )
-    .await;
-
-    // Create stake account to withdraw to
-    let split = Keypair::new();
-    create_blank_stake_account(&mut banks_client, &payer, &recent_blockhash, &split).await;
-    let transaction = Transaction::new_signed_with_payer(
-        &[stake_program::split_only(
-            &user_stake.stake_account,
-            &stake_pool_accounts.staker.pubkey(),
-            1,
-            &split.pubkey(),
-        )],
-        Some(&payer.pubkey()),
-        &[&payer, &stake_pool_accounts.staker],
-        recent_blockhash,
-    );
-
-    banks_client.process_transaction(transaction).await.unwrap();
-
-    let error = stake_pool_accounts
-        .add_validator_to_pool(
-            &mut banks_client,
-            &payer,
-            &recent_blockhash,
-            &user_stake.stake_account,
-        )
-        .await
-        .unwrap()
-        .unwrap();
-    assert_eq!(
-        error,
-        TransactionError::InstructionError(
-            0,
-            InstructionError::Custom(StakePoolError::StakeLamportsNotEqualToMinimum as u32)
-        ),
-    );
-}
-
-#[tokio::test]
-async fn fail_too_much_stake() {
-    let (mut banks_client, payer, recent_blockhash) = program_test().start().await;
-    let stake_pool_accounts = StakePoolAccounts::new();
-    stake_pool_accounts
-        .initialize_stake_pool(&mut banks_client, &payer, &recent_blockhash, 1)
-        .await
-        .unwrap();
-
-    let user_stake = ValidatorStakeAccount::new(&stake_pool_accounts.stake_pool.pubkey());
-    user_stake
-        .create_and_delegate(
-            &mut banks_client,
-            &payer,
-            &recent_blockhash,
-            &stake_pool_accounts.staker,
-        )
-        .await;
-
-    transfer(
-        &mut banks_client,
-        &payer,
-        &recent_blockhash,
-        &user_stake.stake_account,
-        1,
-    )
-    .await;
-
-    let error = stake_pool_accounts
-        .add_validator_to_pool(
-            &mut banks_client,
-            &payer,
-            &recent_blockhash,
-            &user_stake.stake_account,
-        )
-        .await
-        .unwrap()
-        .unwrap();
-    assert_eq!(
-        error,
-        TransactionError::InstructionError(
-            0,
-            InstructionError::Custom(StakePoolError::StakeLamportsNotEqualToMinimum as u32)
-        ),
-    );
-}
-
 #[tokio::test]
 async fn fail_double_add() {
-    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, user_stake) =
+    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, validator_stake) =
         setup().await;
 
     stake_pool_accounts
@@ -273,7 +167,8 @@ async fn fail_double_add() {
             &mut banks_client,
             &payer,
             &recent_blockhash,
-            &user_stake.stake_account,
+            &validator_stake.stake_account,
+            &validator_stake.vote.pubkey(),
         )
         .await;
 
@@ -284,7 +179,8 @@ async fn fail_double_add() {
             &mut banks_client,
             &payer,
             &latest_blockhash,
-            &user_stake.stake_account,
+            &validator_stake.stake_account,
+            &validator_stake.vote.pubkey(),
         )
         .await
         .unwrap();
@@ -303,7 +199,7 @@ async fn fail_double_add() {
 
 #[tokio::test]
 async fn fail_wrong_staker() {
-    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, user_stake) =
+    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, validator_stake) =
         setup().await;
 
     let malicious = Keypair::new();
@@ -313,9 +209,11 @@ async fn fail_wrong_staker() {
             &id(),
             &stake_pool_accounts.stake_pool.pubkey(),
             &malicious.pubkey(),
+            &payer.pubkey(),
             &stake_pool_accounts.withdraw_authority,
             &stake_pool_accounts.validator_list.pubkey(),
-            &user_stake.stake_account,
+            &validator_stake.stake_account,
+            &validator_stake.vote.pubkey(),
         )],
         Some(&payer.pubkey()),
     );
@@ -340,17 +238,22 @@ async fn fail_wrong_staker() {
 
 #[tokio::test]
 async fn fail_without_signature() {
-    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, user_stake) =
+    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, validator_stake) =
         setup().await;
 
     let accounts = vec![
         AccountMeta::new(stake_pool_accounts.stake_pool.pubkey(), false),
         AccountMeta::new_readonly(stake_pool_accounts.staker.pubkey(), false),
+        AccountMeta::new(payer.pubkey(), false),
         AccountMeta::new_readonly(stake_pool_accounts.withdraw_authority, false),
         AccountMeta::new(stake_pool_accounts.validator_list.pubkey(), false),
-        AccountMeta::new(user_stake.stake_account, false),
+        AccountMeta::new(validator_stake.stake_account, false),
+        AccountMeta::new(validator_stake.vote.pubkey(), false),
+        AccountMeta::new_readonly(sysvar::rent::id(), false),
         AccountMeta::new_readonly(sysvar::clock::id(), false),
         AccountMeta::new_readonly(sysvar::stake_history::id(), false),
+        AccountMeta::new_readonly(stake_program::config_id(), false),
+        AccountMeta::new_readonly(system_program::id(), false),
         AccountMeta::new_readonly(stake_program::id(), false),
     ];
     let instruction = Instruction {
@@ -383,20 +286,71 @@ async fn fail_without_signature() {
 
 #[tokio::test]
 async fn fail_with_wrong_stake_program_id() {
-    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, user_stake) =
+    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, validator_stake) =
         setup().await;
 
     let wrong_stake_program = Pubkey::new_unique();
+    let accounts = vec![
+        AccountMeta::new(stake_pool_accounts.stake_pool.pubkey(), false),
+        AccountMeta::new_readonly(stake_pool_accounts.staker.pubkey(), true),
+        AccountMeta::new(payer.pubkey(), true),
+        AccountMeta::new_readonly(stake_pool_accounts.withdraw_authority, false),
+        AccountMeta::new(stake_pool_accounts.validator_list.pubkey(), false),
+        AccountMeta::new(validator_stake.stake_account, false),
+        AccountMeta::new(validator_stake.vote.pubkey(), false),
+        AccountMeta::new_readonly(sysvar::rent::id(), false),
+        AccountMeta::new_readonly(sysvar::clock::id(), false),
+        AccountMeta::new_readonly(sysvar::stake_history::id(), false),
+        AccountMeta::new_readonly(stake_program::config_id(), false),
+        AccountMeta::new_readonly(system_program::id(), false),
+        AccountMeta::new_readonly(wrong_stake_program, false),
+    ];
+    let instruction = Instruction {
+        program_id: id(),
+        accounts,
+        data: instruction::StakePoolInstruction::AddValidatorToPool
+            .try_to_vec()
+            .unwrap(),
+    };
+    let mut transaction = Transaction::new_with_payer(&[instruction], Some(&payer.pubkey()));
+    transaction.sign(&[&payer, &stake_pool_accounts.staker], recent_blockhash);
+    let transaction_error = banks_client
+        .process_transaction(transaction)
+        .await
+        .err()
+        .unwrap();
+
+    match transaction_error {
+        TransportError::TransactionError(TransactionError::InstructionError(_, error)) => {
+            assert_eq!(error, InstructionError::IncorrectProgramId);
+        }
+        _ => panic!(
+            "Wrong error occurs while try to add validator stake account with wrong stake program ID"
+        ),
+    }
+}
+
+#[tokio::test]
+async fn fail_with_wrong_system_program_id() {
+    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, validator_stake) =
+        setup().await;
+
+    let wrong_system_program = Pubkey::new_unique();
 
     let accounts = vec![
         AccountMeta::new(stake_pool_accounts.stake_pool.pubkey(), false),
         AccountMeta::new_readonly(stake_pool_accounts.staker.pubkey(), true),
+        AccountMeta::new(payer.pubkey(), true),
         AccountMeta::new_readonly(stake_pool_accounts.withdraw_authority, false),
         AccountMeta::new(stake_pool_accounts.validator_list.pubkey(), false),
-        AccountMeta::new(user_stake.stake_account, false),
+        AccountMeta::new(validator_stake.stake_account, false),
+        AccountMeta::new(validator_stake.vote.pubkey(), false),
+        AccountMeta::new_readonly(sysvar::rent::id(), false),
         AccountMeta::new_readonly(sysvar::clock::id(), false),
         AccountMeta::new_readonly(sysvar::stake_history::id(), false),
-        AccountMeta::new_readonly(wrong_stake_program, false),
+        AccountMeta::new_readonly(stake_program::config_id(), false),
+        AccountMeta::new_readonly(wrong_system_program, false),
+        AccountMeta::new_readonly(stake_program::id(), false),
     ];
     let instruction = Instruction {
         program_id: id(),
@@ -433,41 +387,43 @@ async fn fail_add_too_many_validator_stake_accounts() {
         .await
         .unwrap();
 
-    let user_stake = ValidatorStakeAccount::new(&stake_pool_accounts.stake_pool.pubkey());
-    user_stake
-        .create_and_delegate(
-            &mut banks_client,
-            &payer,
-            &recent_blockhash,
-            &stake_pool_accounts.staker,
-        )
-        .await;
+    let validator_stake = ValidatorStakeAccount::new(&stake_pool_accounts.stake_pool.pubkey(), 0);
+    create_vote(
+        &mut banks_client,
+        &payer,
+        &recent_blockhash,
+        &validator_stake.validator,
+        &validator_stake.vote,
+    )
+    .await;
 
     let error = stake_pool_accounts
         .add_validator_to_pool(
             &mut banks_client,
             &payer,
             &recent_blockhash,
-            &user_stake.stake_account,
+            &validator_stake.stake_account,
+            &validator_stake.vote.pubkey(),
         )
         .await;
     assert!(error.is_none());
 
-    let user_stake = ValidatorStakeAccount::new(&stake_pool_accounts.stake_pool.pubkey());
-    user_stake
-        .create_and_delegate(
-            &mut banks_client,
-            &payer,
-            &recent_blockhash,
-            &stake_pool_accounts.staker,
-        )
-        .await;
+    let validator_stake = ValidatorStakeAccount::new(&stake_pool_accounts.stake_pool.pubkey(), 0);
+    create_vote(
+        &mut banks_client,
+        &payer,
+        &recent_blockhash,
+        &validator_stake.validator,
+        &validator_stake.vote,
+    )
+    .await;
     let error = stake_pool_accounts
         .add_validator_to_pool(
             &mut banks_client,
             &payer,
             &recent_blockhash,
-            &user_stake.stake_account,
+            &validator_stake.stake_account,
+            &validator_stake.vote.pubkey(),
         )
         .await
         .unwrap()
@@ -483,3 +439,56 @@ async fn fail_with_unupdated_stake_pool() {} // TODO
 
 #[tokio::test]
 async fn fail_with_uninitialized_validator_list_account() {} // TODO
+
+#[tokio::test]
+async fn fail_on_non_vote_account() {
+    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, _) = setup().await;
+
+    let validator = Pubkey::new_unique();
+    let (stake_account, _) =
+        find_stake_program_address(&id(), &validator, &stake_pool_accounts.stake_pool.pubkey());
+
+    let error = stake_pool_accounts
+        .add_validator_to_pool(
+            &mut banks_client,
+            &payer,
+            &recent_blockhash,
+            &stake_account,
+            &validator,
+        )
+        .await
+        .unwrap()
+        .unwrap();
+
+    assert_eq!(
+        error,
+        TransactionError::InstructionError(0, InstructionError::IncorrectProgramId,)
+    );
+}
+
+#[tokio::test]
+async fn fail_on_incorrectly_derived_stake_account() {
+    let (mut banks_client, payer, recent_blockhash, stake_pool_accounts, validator_stake) =
+        setup().await;
+
+    let bad_stake_account = Pubkey::new_unique();
+    let error = stake_pool_accounts
+        .add_validator_to_pool(
+            &mut banks_client,
+            &payer,
+            &recent_blockhash,
+            &bad_stake_account,
+            &validator_stake.vote.pubkey(),
+        )
+        .await
+        .unwrap()
+        .unwrap();
+
+    assert_eq!(
+        error,
+        TransactionError::InstructionError(
+            0,
+            InstructionError::Custom(StakePoolError::InvalidStakeAccountAddress as u32),
+        )
+    );
+}
